/* $Id: order_gui.c 6116 2006-08-25 11:47:36Z rubidium $ */

#include "stdafx.h"
#include "openttd.h"
#include "road_map.h"
#include "station_map.h"
#include "table/sprites.h"
#include "table/strings.h"
#include "functions.h"
#include "map.h"
#include "tile.h"
#include "window.h"
#include "gui.h"
#include "gfx.h"
#include "vehicle.h"
#include "station.h"
#include "town.h"
#include "command.h"
#include "viewport.h"
#include "subsidiaries.h"
#include "depot.h"
#include "waypoint.h"
#include "train.h"
#include "water_map.h"

static int OrderGetSel(const Window *w)
{
	const Vehicle *v = GetVehicle(w->window_number);
	int num = WP(w,order_d).sel;

	return (num >= 0 && num < v->num_orders) ? num : v->num_orders;
}

static void DrawOrdersWindow(Window *w)
{
	const Vehicle *v;
	const Order *order;
	StringID str;
	int sel;
	int y, i;
	bool shared_orders;
	byte color;

	v = GetVehicle(w->window_number);

	w->disabled_state = (IsSisterCompany(v->owner,_local_player)) ? 0 : (
		1 << 4 |   //skip
		1 << 5 |   //delete
		1 << 6 |   //go-to
		1 << 7 |   //loading
		1 << 8 |   //misc
		1 << 9     // % loading options
		);

	if (!_patches.fifo_loading) w->disabled_state |= 1 << 9; //if patch not enabled disable % loading button, misc disabled in the dropdown function
	shared_orders = IsOrderListShared(v);

	if ((uint)v->num_orders + (shared_orders?1:0) <= (uint)WP(w,order_d).sel)
		SETBIT(w->disabled_state, 5); /* delete */

	if (v->num_orders == 0)
		SETBIT(w->disabled_state, 4); /* skip */

	SetVScrollCount(w, v->num_orders + 1);

	sel = OrderGetSel(w);
	SetDParam(2, STR_8827_FULL_LOAD);
	order = GetVehicleOrder(v, sel);

	if (order != NULL) {
		switch (order->type) {
			case OT_GOTO_STATION:
				break;

			case OT_GOTO_DEPOT:
				SETBIT(w->disabled_state, 9);	/* % loading options */
				SetDParam(2,STR_SERVICE);
				w->widget[7].unkA = STR_SERVICE;
				break;

			case OT_GOTO_WAYPOINT:
				SETBIT(w->disabled_state, 7);	/* load options */
				SETBIT(w->disabled_state, 9);	/* % loading options */
				break;

			default:
				SETBIT(w->disabled_state, 7);	/* load options */
				SETBIT(w->disabled_state, 8);	/* misc options */
				SETBIT(w->disabled_state, 9);	/* % loading options */
			break;
		}
	} else {
		SETBIT(w->disabled_state, 7);	/* load options */
		SETBIT(w->disabled_state, 8);	/* misc options */
		SETBIT(w->disabled_state, 9);	/* % loading options */
	}

	SetDParam(0, v->string_id);
	SetDParam(1, v->unitnumber);
	DrawWindowWidgets(w);
	y = 15;

	i = w->vscroll.pos;
	order = GetVehicleOrder(v, i);
	while (order != NULL) {
		str = (v->cur_order_index == i) ? STR_8805 : STR_8804;

		if (i - w->vscroll.pos < w->vscroll.cap) {
			SetDParam(1, 6);
			switch (order->type) {
				case OT_GOTO_STATION:
					// Slight rewrite of the bulky order display
					//	this should allow for a better expansion later

					if(HASBIT(order->flags, OFB_NON_STOP))
					    SetDParam(1, STR_880C_GO_NON_STOP_TO);
					else
						SetDParam(1, STR_8806_GO_TO);
					// Use the originals if only Non stop is set (or not)
					if (order->flags == OF_NON_STOP || order->flags == 0) {
					    SetDParam(2, order->station);
					} else {
					    SetDParam(1, STR_NEW_GO_TO);

					    // Non-Stop
					    if(HASBIT(order->flags, OFB_NON_STOP))
						SetDParam(1, STR_NEW_GO_NON_STOP_TO);

					    SetDParam(2, order->station);

						// Transfer
						if(HASBIT(order->flags, OFB_TRANSFER))
							SetDParam(3, STR_ORDER_TRANSFER);

					    // Load
					    if(HASBIT(order->flags, OFB_FULL_LOAD))
							SetDParam(3, STR_ORDER_LOAD + (HASBIT(order->flags, OFB_EXACT_LOAD)?1:0)+ (HASBIT(order->flags, OFB_TRANSFER)?2:0));

					    // Unload
					    else if(HASBIT(order->flags, OFB_UNLOAD))
							SetDParam(3, STR_ORDER_UNLOAD + (HASBIT(order->flags, OFB_EXACT_LOAD)?1:0) + (HASBIT(order->flags, OFB_TRANSFER)?2:0));

					    // The percentile of load
					    if(HASBIT(order->flags, OFB_UNLOAD) || HASBIT(order->flags, OFB_FULL_LOAD)) {
							if(order->percentilefill)
								SetDParam(4, STR_ORDER_10 + (order->percentilefill / 10) -1);
							else
								SetDParam(4, STR_ORDER_10 + 9);	// 100% load
					    }

						if(HASBIT(order->flags, OFB_CARGO_LOAD)){
							SetDParam(5, STR_ORDER_LOAD_PER_TRAIN);
						} else {
							SetDParam(5, STR_ORDER_LOAD_PER_CARGO);
						}
						if(HASBIT(order->flags, OFB_PRIORITY)){
							SetDParam(6, STR_ORDER_PRIORITY);
						} else {
							SetDParam(6, STR_EMPTY);
						}
					}
					break;

				case OT_GOTO_DEPOT: {
					StringID s = STR_NULL;
					if (v->type == VEH_Aircraft) {
						s = STR_GO_TO_AIRPORT_HANGAR;
						SetDParam(2, order->station);
					} else {
						SetDParam(2, GetDepot(order->station)->town_index);

						switch (v->type) {
							case VEH_Train: s = (order->flags & OF_NON_STOP) ? STR_880F_GO_NON_STOP_TO_TRAIN_DEPOT : STR_GO_TO_TRAIN_DEPOT; break;
							case VEH_Road:  s = STR_9038_GO_TO_ROADVEH_DEPOT; break;
							case VEH_Ship:  s = STR_GO_TO_SHIP_DEPOT; break;
							default: break;
						}
					}

					if (order->flags & OF_FULL_LOAD) s++; /* service at */

					SetDParam(1, s);
					break;
				}

				case OT_GOTO_WAYPOINT:
					SetDParam(1, (order->flags & OF_NON_STOP) ? STR_GO_NON_STOP_TO_WAYPOINT : STR_GO_TO_WAYPOINT);
					SetDParam(2, order->station);
					break;

				default: break;
			}
			color = (i == WP(w,order_d).sel) ? 0xC : 0x10;
			SetDParam(0, i + 1);
			if (order->type != OT_DUMMY) {
				DrawString(2, y, str, color);
			} else {
				SetDParam(1, STR_INVALID_ORDER);
				SetDParam(2, order->station);
				DrawString(2, y, str, color);
			}
			y += 10;
		}
		i++;
		order = order->next;
	}

	if (i - w->vscroll.pos < w->vscroll.cap) {
		str = shared_orders ? STR_END_OF_SHARED_ORDERS : STR_882A_END_OF_ORDERS;
		color = (i == WP(w,order_d).sel) ? 0xC : 0x10;
		DrawString(2, y, str, color);
	}
}

static Order GetOrderCmdFromTile(const Vehicle *v, TileIndex tile)
{
	Order order;
	order.next  = NULL;
	order.index = 0;

	// check depot first
	if (_patches.gotodepot) {
		switch (GetTileType(tile)) {
		case MP_RAILWAY:
 			if (v->type == VEH_Train &&
 					(IsSisterCompany(GetTileOwner(tile), _local_player) || _patches.shared_depots)) {
				if (IsRailDepot(tile)) {
					order.type = OT_GOTO_DEPOT;
					order.flags = OF_PART_OF_ORDERS;
					order.station = GetDepotByTile(tile)->index;
					return order;
				}
			}
			break;

		case MP_STREET:
 			if (GetRoadTileType(tile) == ROAD_TILE_DEPOT && v->type == VEH_Road &&
 					(IsSisterCompany(GetTileOwner(tile), _local_player) || _patches.shared_depots)) {
				order.type = OT_GOTO_DEPOT;
				order.flags = OF_PART_OF_ORDERS;
				order.station = GetDepotByTile(tile)->index;
				return order;
			}
			break;

		case MP_STATION:
			if (v->type != VEH_Aircraft) break;
			if (IsHangar(tile) &&
					(IsSisterCompany(GetTileOwner(tile), _local_player) || _patches.shared_depots)) {
				order.type = OT_GOTO_DEPOT;
				order.flags = OF_PART_OF_ORDERS;
				order.station = GetStationIndex(tile);
				return order;
			}
			break;

		case MP_WATER:
			if (v->type != VEH_Ship) break;
			if (IsTileDepotType(tile, TRANSPORT_WATER) &&
					(IsSisterCompany(GetTileOwner(tile), _local_player) || _patches.shared_depots)) {
				TileIndex tile2 = GetOtherShipDepotTile(tile);

				order.type = OT_GOTO_DEPOT;
				order.flags = OF_PART_OF_ORDERS;
				order.station = GetDepotByTile(tile < tile2 ? tile : tile2)->index;
				return order;
			}

			default:
				break;
		}
	}

	// check waypoint
	if (IsTileType(tile, MP_RAILWAY) &&
			v->type == VEH_Train &&
			(IsSisterCompany(GetTileOwner(tile), _local_player) || _patches.shared_stations) &&
			IsRailWaypoint(tile)) {
		order.type = OT_GOTO_WAYPOINT;
		order.flags = 0;
		order.station = GetWaypointByTile(tile)->index;
		return order;
	}

	if (IsTileType(tile, MP_STATION)) {
		StationID st_index = GetStationIndex(tile);
		const Station *st = GetStation(st_index);

		if (IsSisterCompany(st->owner, _current_player) || _patches.shared_stations || st->owner == OWNER_NONE) {
			byte facil;
			(facil=FACIL_DOCK, v->type == VEH_Ship) ||
			(facil=FACIL_TRAIN, v->type == VEH_Train) ||
			(facil=FACIL_AIRPORT, v->type == VEH_Aircraft) ||
			(facil=FACIL_BUS_STOP, v->type == VEH_Road && v->cargo_type == CT_PASSENGERS) ||
			(facil=FACIL_TRUCK_STOP, 1);
			if (st->facilities & facil) {
				order.type = OT_GOTO_STATION;
				order.flags = 0;
				order.station = st_index;
				return order;
			}
		}
	}

	// not found
	order.type = OT_NOTHING;
	order.flags = 0;
	order.station = INVALID_STATION;
	return order;
}

static bool HandleOrderVehClick(const Vehicle *v, const Vehicle *u, Window *w)
{
	if (u->type != v->type) return false;

	if (u->type == VEH_Train && !IsFrontEngine(u)) {
		u = GetFirstVehicleInChain(u);
		if (!IsFrontEngine(u)) return false;
	}

	// v is vehicle getting orders. Only copy/clone orders if vehicle doesn't have any orders yet
	// obviously if you press CTRL on a non-empty orders vehicle you know what you are doing
	if (v->num_orders != 0 && _ctrl_pressed == 0) return false;

	if (DoCommandP(v->tile, v->index | (u->index << 16), _ctrl_pressed ? 0 : 1, NULL,
		_ctrl_pressed ? CMD_CLONE_ORDER | CMD_MSG(STR_CANT_SHARE_ORDER_LIST) : CMD_CLONE_ORDER | CMD_MSG(STR_CANT_COPY_ORDER_LIST))) {
		WP(w,order_d).sel = -1;
		ResetObjectToPlace();
	}

	return true;
}

static void OrdersPlaceObj(const Vehicle *v, TileIndex tile, Window *w)
{
	Order cmd;
	const Vehicle *u;

	// check if we're clicking on a vehicle first.. clone orders in that case.
	u = CheckMouseOverVehicle();
	if (u != NULL && HandleOrderVehClick(v, u, w)) return;

	cmd = GetOrderCmdFromTile(v, tile);
	if (cmd.type == OT_NOTHING) return;

	if (DoCommandP(v->tile, v->index + (OrderGetSel(w) << 16), PackOrder(&cmd), NULL, CMD_INSERT_ORDER | CMD_MSG(STR_8833_CAN_T_INSERT_NEW_ORDER))) {
		if (WP(w,order_d).sel != -1) WP(w,order_d).sel++;
		ResetObjectToPlace();
	}
}

static void OrderClick_Goto(Window *w, const Vehicle *v)
{
	InvalidateWidget(w, 6);
	TOGGLEBIT(w->click_state, 6);
	if (HASBIT(w->click_state, 6)) {
		_place_clicked_vehicle = NULL;
		SetObjectToPlaceWnd(ANIMCURSOR_PICKSTATION, 1, w);
	} else {
		ResetObjectToPlace();
	}
}

static void OrderClick_FullLoad(Window *w, const Vehicle *v)
{
	DoCommandP(v->tile, v->index + (OrderGetSel(w) << 16), OFB_FULL_LOAD, NULL, CMD_MODIFY_ORDER | CMD_MSG(STR_8835_CAN_T_MODIFY_THIS_ORDER));
}

static void OrderClick_Unload(Window *w, const Vehicle *v)
{
	DoCommandP(v->tile, v->index + (OrderGetSel(w) << 16), OFB_UNLOAD,    NULL, CMD_MODIFY_ORDER | CMD_MSG(STR_8835_CAN_T_MODIFY_THIS_ORDER));
}

static void OrderClick_Nonstop(Window *w, const Vehicle *v)
{
	DoCommandP(v->tile, v->index + (OrderGetSel(w) << 16), OFB_NON_STOP,  NULL, CMD_MODIFY_ORDER | CMD_MSG(STR_8835_CAN_T_MODIFY_THIS_ORDER));
}

static void OrderClick_Transfer(Window* w, const Vehicle* v)
{
	DoCommandP(v->tile, v->index + (OrderGetSel(w) <<  16), OFB_TRANSFER, NULL, CMD_MODIFY_ORDER | CMD_MSG(STR_8835_CAN_T_MODIFY_THIS_ORDER));
}

static void OrderClick_Skip(Window *w, const Vehicle *v)
{
	DoCommandP(v->tile, v->index, 0, NULL, CMD_SKIP_ORDER);
}

static void OrderClick_Delete(Window *w, const Vehicle *v)
{
	DoCommandP(v->tile, v->index, OrderGetSel(w), NULL, CMD_DELETE_ORDER | CMD_MSG(STR_8834_CAN_T_DELETE_THIS_ORDER));
}

static void OrderClick_LoadPercent(Window *w, const Vehicle *v, int percent)
{
	DoCommandP(v->tile, v->index + (OrderGetSel(w) << 16), OFB_PERCENT | percent << 16, NULL, CMD_MODIFY_ORDER | CMD_MSG(STR_8835_CAN_T_MODIFY_THIS_ORDER));
}

static void OrderClick_LoadExact(Window *w, const Vehicle *v)
{
	DoCommandP(v->tile, v->index + (OrderGetSel(w) << 16), OFB_EXACT_LOAD,    NULL, CMD_MODIFY_ORDER | CMD_MSG(STR_8835_CAN_T_MODIFY_THIS_ORDER));
}
static void OrderClick_Priority(Window *w,const Vehicle *v)
{
	DoCommandP(v->tile, v->index + (OrderGetSel(w) << 16), OFB_PRIORITY,    NULL, CMD_MODIFY_ORDER | CMD_MSG(STR_8835_CAN_T_MODIFY_THIS_ORDER));
}
static void OrderClick_CargoLoad(Window *w, const Vehicle *v)
{
	DoCommandP(v->tile, v->index + (OrderGetSel(w) << 16), OFB_CARGO_LOAD,    NULL, CMD_MODIFY_ORDER | CMD_MSG(STR_8835_CAN_T_MODIFY_THIS_ORDER));
}


typedef void OnButtonVehClick(Window *w, const Vehicle *v);

static OnButtonVehClick* const _order_button_proc[] = {
	OrderClick_Skip,
	OrderClick_Delete,
	OrderClick_Nonstop,
	OrderClick_Goto,
	OrderClick_FullLoad,
	OrderClick_LoadExact,
	OrderClick_Unload,
	OrderClick_Transfer,
	OrderClick_Priority,
	OrderClick_CargoLoad,
};

static const uint16 _order_keycodes[] = {
	'D', //skip order
	'F', //delete order
	'G', //non-stop
	'H', //goto order
	'J', //full load
	0, //load exact gets no key
	'K'  //unload
};

static void OrdersWndProc(Window *w, WindowEvent *e)
{
	switch (e->event) {
	case WE_PAINT:
		DrawOrdersWindow(w);
		break;

	case WE_CLICK: {
		Vehicle *v = GetVehicle(w->window_number);
		switch (e->click.widget) {
		case 2: { /* orders list */
			int sel = (e->click.pt.y - 15) / 10;

			if ((uint)sel >= w->vscroll.cap) return;

			sel += w->vscroll.pos;

			if (_ctrl_pressed && sel < v->num_orders) {
				const Order *ord = GetVehicleOrder(v, sel);
				TileIndex xy;

				switch (ord->type) {
					case OT_GOTO_STATION:  xy = GetStation(ord->station)->xy ; break;
					case OT_GOTO_DEPOT:    xy = GetDepot(ord->station)->xy;    break;
					case OT_GOTO_WAYPOINT: xy = GetWaypoint(ord->station)->xy; break;
					default:               xy = 0; break;
				}

				if (xy != 0) ScrollMainWindowToTile(xy);
				return;
			}

			if (sel == WP(w,order_d).sel) sel = -1;
			WP(w,order_d).sel = sel;
			SetWindowDirty(w);
		}	break;

		case 4: /* skip button */
			OrderClick_Skip(w, v);
			break;

		case 5: /* delete button */
			OrderClick_Delete(w, v);
			break;


		case 6: /* goto button */
			OrderClick_Goto(w, v);
			break;

		case 7: /* load drop down */
			{
				static const StringID loading_buttons[] = {
				STR_8827_FULL_LOAD,
				STR_8828_UNLOAD,
				STR_886F_TRANSFER,
				INVALID_STRING_ID};
				int sel = WP(w,order_d).sel;
				const Order *ord = GetVehicleOrder(v, sel);
				if (ord->type == OT_GOTO_DEPOT) {
					OrderClick_FullLoad(w, v);
				} else {
					ShowDropDownMenu(w,loading_buttons,-1,7,0,0);
				}
			}
			break;

		case 8: /* misc drop down */
			{
				uint32 disabled_btn = 0,hidden_btn = 0;

				static const StringID misc_buttons[] = {
				STR_8825_NON_STOP,
				STR_ORDER_LOAD_EXACT,
				STR_ORDER_PRIORITY_BTN,
				STR_ORDER_LOAD_PER_BTN,
				INVALID_STRING_ID};
				int sel = WP(w,order_d).sel;
				const Order *ord = GetVehicleOrder(v, sel);
				if (!_patches.fifo_loading) hidden_btn |= ( 1 << 1 | 1 << 2 | 1 << 3 ); //hide all buttons exept non-stop
				if (ord->percentilefill == 0 || ord->percentilefill == 100) disabled_btn |= 1 << 1; //disable load exact button if no percential loading
				if (!HASBIT(ord->flags, OFB_FULL_LOAD)) disabled_btn |= 1 << 2 | 1 << 3; //disable the last 2 buttons if no loading job
				ShowDropDownMenu(w,misc_buttons,-1,8,disabled_btn,hidden_btn);

			}
			break;

		case 9: /* % load drop down */
			{
				static const StringID percent_buttons[] = {
				STR_ORDER_10,
				STR_ORDER_20,
				STR_ORDER_30,
				STR_ORDER_40,
				STR_ORDER_50,
				STR_ORDER_60,
				STR_ORDER_70,
				STR_ORDER_80,
				STR_ORDER_90,
				STR_ORDER_100,
				INVALID_STRING_ID};

				ShowDropDownMenu(w,percent_buttons,-1,9,0,0);
			}
			break;
		}
	} break;

	case WE_KEYPRESS: {
		Vehicle *v = GetVehicle(w->window_number);
		uint i;

		for (i = 0; i < lengthof(_order_keycodes); i++) {
			if (e->keypress.keycode == _order_keycodes[i]) {
				e->keypress.cont = false;
				//see if the button is disabled
				if (!HASBIT(w->disabled_state, i + 4)) _order_button_proc[i](w, v);
				break;
			}
		}
		break;
	}

	case WE_RCLICK: {
		const Vehicle *v = GetVehicle(w->window_number);
		int s = OrderGetSel(w);

		if (e->click.widget != 8) break;
		if (s == v->num_orders || GetVehicleOrder(v, s)->type != OT_GOTO_DEPOT) {
			GuiShowTooltips(STR_8857_MAKE_THE_HIGHLIGHTED_ORDER);
		} else {
			GuiShowTooltips(STR_SERVICE_HINT);
		}
	} break;

	case WE_PLACE_OBJ: {
		OrdersPlaceObj(GetVehicle(w->window_number), e->place.tile, w);
	} break;

	case WE_ABORT_PLACE_OBJ: {
		CLRBIT(w->click_state, 6);
		InvalidateWidget(w, 6);
	} break;

	// check if a vehicle in a depot was clicked..
	case WE_MOUSELOOP: {
		const Vehicle *v = _place_clicked_vehicle;
		/*
		 * Check if we clicked on a vehicle
		 * and if the GOTO button of this window is pressed
		 * This is because of all open order windows WE_MOUSELOOP is called
		 * and if you have 3 windows open, and this check is not done
		 * the order is copied to the last open window instead of the
		 * one where GOTO is enalbed
		 */
		if (v != NULL && HASBIT(w->click_state, 7)) {
			_place_clicked_vehicle = NULL;
			HandleOrderVehClick(GetVehicle(w->window_number), v, w);
		}
	} break;

	case WE_RESIZE:
		/* Update the scroll + matrix */
		w->vscroll.cap = (w->widget[2].bottom - w->widget[2].top) / 10;
		break;
	case WE_DROPDOWN_SELECT: {
		Vehicle *v = GetVehicle(w->window_number);
		switch(e->dropdown.button){
			case 7: {
				switch(e->dropdown.index) {
					case 0: OrderClick_FullLoad(w, v); break;
					case 1: OrderClick_Unload(w, v); break;
					case 2: OrderClick_Transfer(w, v); break;
				} break;
			} break;
			case 8: {
				switch(e->dropdown.index) {
					case 0: OrderClick_Nonstop(w, v); break;
					case 1: OrderClick_LoadExact(w, v); break;
					case 2: OrderClick_Priority(w, v); break;
					case 3: OrderClick_CargoLoad(w, v); break;
				} break;
			} break;
			case 9: {//0-1 so 0=10%
				OrderClick_LoadPercent(w, v, e->dropdown.index);
				break;
			} break;
		}
	} break; //break of case WE_DROPDOWN_SELECT:
	} //end of switch
}

static const Widget _orders_train_widgets[] = {
{   WWT_CLOSEBOX,   RESIZE_NONE,    14,     0,    10,     0,    13, STR_00C5,                STR_018B_CLOSE_WINDOW},
{    WWT_CAPTION,   RESIZE_RIGHT,   14,    11,   384,     0,    13, STR_8829_ORDERS,         STR_018C_WINDOW_TITLE_DRAG_THIS},
{      WWT_PANEL,   RESIZE_RB,      14,     0,   372,    14,    75, 0x0,                     STR_8852_ORDERS_LIST_CLICK_ON_ORDER},
{  WWT_SCROLLBAR,   RESIZE_LRB,     14,   373,   384,    14,    75, 0x0,                     STR_0190_SCROLL_BAR_SCROLLS_LIST},
{ WWT_PUSHTXTBTN,   RESIZE_TB,      14,     0,    62,    76,    87, STR_8823_SKIP,           STR_8853_SKIP_THE_CURRENT_ORDER},
{ WWT_PUSHTXTBTN,   RESIZE_TB,      14,    63,   124,    76,    87, STR_8824_DELETE,         STR_8854_DELETE_THE_HIGHLIGHTED},
{WWT_NODISTXTBTN,   RESIZE_TB,      14,   125,   186,    76,    87, STR_8826_GO_TO,          STR_8856_INSERT_A_NEW_ORDER_BEFORE},

{    WWT_TEXTBTN,   RESIZE_TB,      14,   187,   248,    76,    87, STR_LOADING_BTN,         STR_NULL},
{    WWT_TEXTBTN,   RESIZE_TB,      14,   249,   310,    76,    87, STR_ORDER_MISC_BTN,      STR_8858_MAKE_THE_HIGHLIGHTED_ORDER},
{    WWT_TEXTBTN,   RESIZE_TB,      14,   311,   372,    76,    87, STR_ORDER_PERCENT_BTN,   STR_ORDER_PERCENT_HINT},

{      WWT_PANEL,   RESIZE_RTB,     14,   373,   372,    76,    87, 0x0,                     STR_NULL},
{  WWT_RESIZEBOX,   RESIZE_LRTB,    14,   373,   384,    76,    87, 0x0,                     STR_RESIZE_BUTTON},
{   WIDGETS_END},
};

static const WindowDesc _orders_train_desc = {
	-1,-1, 385, 88,
	WC_VEHICLE_ORDERS,WC_VEHICLE_VIEW,
	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS | WDF_RESIZABLE,
	_orders_train_widgets,
	OrdersWndProc
};

static const Widget _orders_widgets[] = {
{   WWT_CLOSEBOX,   RESIZE_NONE,    14,     0,    10,     0,    13, STR_00C5,                STR_018B_CLOSE_WINDOW},
{    WWT_CAPTION,   RESIZE_RIGHT,   14,    11,   384,     0,    13, STR_8829_ORDERS,         STR_018C_WINDOW_TITLE_DRAG_THIS},
{      WWT_PANEL,   RESIZE_RB,      14,     0,   383,    14,    75, 0x0,                     STR_8852_ORDERS_LIST_CLICK_ON_ORDER},
{  WWT_SCROLLBAR,   RESIZE_LRB,     14,   373,   384,    14,    75, 0x0,                     STR_0190_SCROLL_BAR_SCROLLS_LIST},
{ WWT_PUSHTXTBTN,   RESIZE_TB,      14,     0,    62,    76,    87, STR_8823_SKIP,           STR_8853_SKIP_THE_CURRENT_ORDER},
{ WWT_PUSHTXTBTN,   RESIZE_TB,      14,    63,   128,    76,    87, STR_8824_DELETE,         STR_8854_DELETE_THE_HIGHLIGHTED},
{WWT_NODISTXTBTN,   RESIZE_TB,      14,   129,   186,    76,    87, STR_8826_GO_TO,          STR_8856_INSERT_A_NEW_ORDER_BEFORE},

{    WWT_TEXTBTN,   RESIZE_TB,      14,   187,   248,    76,    87, STR_LOADING_BTN,         STR_NULL},
{    WWT_TEXTBTN,   RESIZE_TB,      14,   249,   310,    76,    87, STR_ORDER_MISC_BTN,      STR_8858_MAKE_THE_HIGHLIGHTED_ORDER},
{    WWT_TEXTBTN,   RESIZE_TB,      14,   311,   372,    76,    87, STR_ORDER_PERCENT_BTN,   STR_886D_MAKE_THE_HIGHLIGHTED_ORDER},

{      WWT_PANEL,   RESIZE_RTB,     14,   373,   372,    76,    87, 0x0,                     STR_NULL},
{  WWT_RESIZEBOX,   RESIZE_LRTB,    14,   373,   384,    76,    87, 0x0,                     STR_RESIZE_BUTTON},
{   WIDGETS_END},
};

static const WindowDesc _orders_desc = {
	-1,-1, 385, 88,
	WC_VEHICLE_ORDERS,WC_VEHICLE_VIEW,
	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS | WDF_RESIZABLE,
	_orders_widgets,
	OrdersWndProc
};

static const Widget _other_orders_widgets[] = {
{   WWT_CLOSEBOX,   RESIZE_NONE,    14,     0,    10,     0,    13, STR_00C5,        STR_018B_CLOSE_WINDOW},
{    WWT_CAPTION,   RESIZE_RIGHT,   14,    11,   331,     0,    13, STR_A00B_ORDERS, STR_018C_WINDOW_TITLE_DRAG_THIS},
{      WWT_PANEL,   RESIZE_RB,      14,     0,   319,    14,    75, 0x0,             STR_8852_ORDERS_LIST_CLICK_ON_ORDER},
{  WWT_SCROLLBAR,   RESIZE_LRB,     14,   320,   331,    14,    75, 0x0,             STR_0190_SCROLL_BAR_SCROLLS_LIST},
{      WWT_EMPTY,   RESIZE_NONE,    14,     0,   319,    76,    87, 0x0,             STR_NULL},
{      WWT_EMPTY,   RESIZE_NONE,    14,     0,   319,    76,    87, 0x0,             STR_NULL},
{      WWT_EMPTY,   RESIZE_NONE,    14,     0,   319,    76,    87, 0x0,             STR_NULL},
{      WWT_EMPTY,   RESIZE_NONE,    14,     0,   319,    76,    87, 0x0,             STR_NULL},
{      WWT_EMPTY,   RESIZE_NONE,    14,     0,   319,    76,    87, 0x0,             STR_NULL},
{      WWT_EMPTY,   RESIZE_NONE,    14,     0,   319,    76,    87, 0x0,             STR_NULL},
{      WWT_PANEL,   RESIZE_RTB,     14,     0,   319,    76,    87, 0x0,             STR_NULL},
{  WWT_RESIZEBOX,   RESIZE_LRTB,    14,   320,   331,    76,    87, 0x0,             STR_RESIZE_BUTTON},
{   WIDGETS_END},
};

static const WindowDesc _other_orders_desc = {
	-1,-1, 332, 88,
	WC_VEHICLE_ORDERS,WC_VEHICLE_VIEW,
	WDF_STD_TOOLTIPS | WDF_STD_BTN | WDF_DEF_WIDGET | WDF_UNCLICK_BUTTONS | WDF_RESIZABLE,
	_other_orders_widgets,
	OrdersWndProc
};

void ShowOrdersWindow(const Vehicle *v)
{
	Window *w;
	VehicleID veh = v->index;

	DeleteWindowById(WC_VEHICLE_ORDERS, veh);
	DeleteWindowById(WC_VEHICLE_DETAILS, veh);

	_alloc_wnd_parent_num = veh;

	if (!IsSisterCompany(v->owner, _local_player)) {
		w = AllocateWindowDesc(&_other_orders_desc);
	} else {
		w = AllocateWindowDesc((v->type == VEH_Train) ? &_orders_train_desc : &_orders_desc);
	}

	if (w != NULL) {
		w->window_number = veh;
		w->caption_color = v->owner;
		w->vscroll.cap = 6;
		w->resize.step_height = 10;
		WP(w,order_d).sel = -1;
	}
}
