/* $Id: road_map.c 5829 2006-08-09 23:03:34Z richk $ */

#include "stdafx.h"
#include "openttd.h"
#include "bridge_map.h"
#include "functions.h"
#include "road_map.h"
#include "station.h"
#include "tunnel_map.h"
#include "station_map.h"
#include "depot.h"


RoadBits GetAnyRoadBits(TileIndex tile)
{
	switch (GetTileType(tile)) {
		case MP_STREET:
			switch (GetRoadTileType(tile)) {
				default:
				case ROAD_TILE_NORMAL:   return GetRoadBits(tile);
				case ROAD_TILE_CROSSING: return GetCrossingRoadBits(tile);
				case ROAD_TILE_DEPOT:    return DiagDirToRoadBits(GetRoadDepotDirection(tile));
			}

		case MP_STATION:
			if (!IsRoadStopTile(tile)) return 0;
			if (IsDriveThroughStopTile(tile)) return (GetRoadStopDir(tile) == DIAGDIR_NE) ? ROAD_X : ROAD_Y;
			return DiagDirToRoadBits(GetRoadStopDir(tile));

		case MP_TUNNELBRIDGE:
			if (IsBridge(tile)) {
				if (IsBridgeMiddle(tile)) {
					if (!IsTransportUnderBridge(tile) ||
							GetBridgeTransportType(tile) != TRANSPORT_ROAD) {
						return 0;
					}
					return GetRoadBitsUnderBridge(tile);
				} else {
					// ending
					if (GetBridgeTransportType(tile) != TRANSPORT_ROAD) return 0;
					return DiagDirToRoadBits(ReverseDiagDir(GetBridgeRampDirection(tile)));
				}
			} else {
				// tunnel
				if (GetTunnelTransportType(tile) != TRANSPORT_ROAD) return 0;
				return DiagDirToRoadBits(ReverseDiagDir(GetTunnelDirection(tile)));
			}

		default: return 0;
	}
}


TrackBits GetAnyRoadTrackBits(TileIndex tile)
{
	uint32 r;

	// Don't allow building through road depot tiles.
	if (IsTileType(tile, MP_STREET) && IsTileDepotType(tile, TRANSPORT_ROAD)) {
		return 0;
	}

	r = GetTileTrackStatus(tile, TRANSPORT_ROAD);
	return (byte)(r | (r >> 8));
}


LevelCrossingType GetLevelCrossingType(TileIndex t, Trackdir trackdir, TileIndex *other_half)
{
	Axis road_axis;
	TrackBits trackbits;
	TileIndex prev_tile, next_tile;

	if (!IsLevelCrossingTile(t)) return NOT_CROSSING;

	trackbits = GetCrossingRailBits(t);
	if (trackbits == TRACK_BIT_X || trackbits == TRACK_BIT_Y) {
		return SINGLE_CROSSING;
	}

	road_axis = GetCrossingRoadAxis(t);
	next_tile = RailLookAhead(t, trackdir, 1);
	if (next_tile != 0 && IsLevelCrossingTile(next_tile) &&
			DiagDirToAxis(TrackdirToExitdir(trackdir)) == road_axis &&
			GetCrossingRoadAxis(next_tile) == road_axis) {
		*other_half = next_tile;
		return DOUBLE_CROSSING_FIRST_HALF;
	}

	prev_tile = RailLookAhead(t, ReverseTrackdir(trackdir), 1);
	if (prev_tile != 0 && IsLevelCrossingTile(prev_tile) &&
			DiagDirToAxis(TrackdirToExitdir(ReverseTrackdir(trackdir))) == road_axis &&
			GetCrossingRoadAxis(prev_tile) == road_axis) {
		*other_half = prev_tile;
		return DOUBLE_CROSSING_SECOND_HALF;
	}
	return SINGLE_CROSSING;
}
