#include "stdafx.h"
#include "openttd.h"
#include "player.h"
#include "command.h"
#include "subsidiaries.h"
#include "gfx.h"
#include "functions.h"
#include "variables.h"
#include "table/strings.h"

extern void ShowErrorMessage(StringID msg_1, StringID msg_2, int x, int y);

int32 CmdSubsidiaryBuyShareResponse(TileIndex tile, uint32 flags, uint32 p1, uint32 p2)
{

	Player *p;
	Player *player2;
	int motherIndex;
	Player *mother;
	int64 cost;
	byte *b;
	int i;
	PlayerID old_cp;

	p = GetPlayer(_current_player);
	cost = CalculateCompanyValue(GetPlayer(_current_player)) >> 2;

	if(flags & DC_EXEC){
		if(p2 == 0){
			if(p1 == _local_player){
				ShowErrorMessage(STR_SUBSIDIARY_SELL_SHARE_DENIED, STR_NULL ,0,0);
			}
			return 0;
		}

		b = p->share_owners;
		while (*b != 0xFF) b++; /* share owners is guaranteed to contain at least one 0xFF */
		*b = GetMotherCompany(GetPlayer(p1));

		for(i=0;p->share_owners[i] == GetMotherCompany(GetPlayer(p1));) {
			if (++i == 4) {
				// The buying company is taking over this one.
				p->is_ai = 0;
				motherIndex = GetMotherCompany(GetPlayer(p1));

				/* The bought out company must surrender its shares into other companies, to the */
				/* buying one. */
				FOR_ALL_PLAYERS(player2){
					for(i = 0; i < 4; i++){
						if(player2->share_owners[i] == p->index)
							player2->share_owners[i] = motherIndex;
					}
				}

				/* At this point, due to the share exchanges, it is possible that the buying company */
				/* holds shares of itself, clean the mess */
				mother = GetPlayer(motherIndex);
				for(i = 0; i < 4; i ++){
					if(mother->share_owners[i] == motherIndex)
						mother->share_owners[i] = 0xFF;
				}

				MarkWholeScreenDirty();
				break;
			}
		}
		InvalidateWindow(WC_COMPANY, (int)p1);
		if(p1 == _local_player){
			//HACK!!!!
			//Should use network message etc.
			ShowErrorMessage(STR_SUBSIDIARY_SELL_SHARE_GRANTED, STR_NULL ,0,0);
		}
		old_cp = _current_player;
		_current_player = p1;
		SubtractMoneyFromPlayer(cost);
		_current_player = old_cp;
	}
//		return cost;
	return 0;
}

int32 CmdSubsidiarySendMoneyOnRequest(TileIndex tile, uint32 flags, uint32 p1, uint32 p2)
{
	PlayerID old_cp;
	SET_EXPENSES_TYPE(EXPENSES_OTHER);

	if((int32)p2 == -1){
		// request denied
		if(_local_player == p1){
			ShowErrorMessage(STR_SUBSIDIARY_CASH_REQUEST_DENIED,STR_NULL,0,0);
		}
		return 0;
	}

	if(_local_player == p1){
		ShowErrorMessage(STR_SUBSIDIARY_CASH_REQUEST_GRANTED,STR_NULL,0,0);
	}

	// Add money to player
	if(flags & DC_EXEC){
		old_cp = _current_player;
		_current_player = p1;
		SubtractMoneyFromPlayer(-(int32)p2 + (p2 / 16)); // don't forget to correct for tax
		_current_player = old_cp;
	}

	// Subtract money from local-player
	return (int32)p2;
}

// p1 = destination
// p2 = amount
extern void AskSendMoney(PlayerID player,int32 amount);
int32 CmdSubsidiaryGetMoney(TileIndex tile, uint32 flags, uint32 p1, uint32 p2)
{
	if(_local_player == p1){
		AskSendMoney(_current_player,p2);
	}
	return 0;
}

// p1 = destination
// p2 = amount
int32 CmdSubsidiarySendMoney(TileIndex tile, uint32 flags, uint32 p1, uint32 p2)
{
	PlayerID old_cp;
	SET_EXPENSES_TYPE(EXPENSES_OTHER);

	// Add money to player
	if(flags & DC_EXEC){
		old_cp = _current_player;
		_current_player = p1;
		SubtractMoneyFromPlayer(-(int32)p2 + (p2 / 16)); // don't forget to correct for tax
		_current_player = old_cp;
	}

	// Subtract money from local-player
	return (int32)p2;
}

